
#? ['raise']
raise

#? ['Exception']
except

#? []
b + continu

#? []
b + continue

#? ['continue']
b; continue

#? ['continue']
b; continu

#? []
c + brea

#? []
a + break

#? ['break']
b; break

# -----------------
# Keywords should not appear everywhere.
# -----------------

#? []
with open() as f
#? []
def i
#? []
class i

#? []
continue i

# More syntax details, e.g. while only after newline, but not after semicolon,
# continue also after semicolon
#? ['while']
while
#? []
x while
#? []
x; while
#? ['continue']
x; continue

#? []
and
#? ['and']
x and
#? []
x * and



# -*- coding: utf-8 -*-
"""
All character set and unicode related tests.
"""
from jedi import Script
from jedi._compatibility import u, unicode


def test_unicode_script():
    """ normally no unicode objects are being used. (<=2.7) """
    s = unicode("import datetime; datetime.timedelta")
    completions = Script(s).completions()
    assert len(completions)
    assert type(completions[0].description) is unicode

    s = u("author='öä'; author")
    completions = Script(s).completions()
    x = completions[0].description
    assert type(x) is unicode

    s = u("#-*- coding: iso-8859-1 -*-\nauthor='öä'; author")
    s = s.encode('latin-1')
    completions = Script(s).completions()
    assert type(completions[0].description) is unicode


def test_unicode_attribute():
    """ github jedi-vim issue #94 """
    s1 = u('#-*- coding: utf-8 -*-\nclass Person():\n'
           '    name = "e"\n\nPerson().name.')
    completions1 = Script(s1).completions()
    assert 'strip' in [c.name for c in completions1]
    s2 = u('#-*- coding: utf-8 -*-\nclass Person():\n'
           '    name = "é"\n\nPerson().name.')
    completions2 = Script(s2).completions()
    assert 'strip' in [c.name for c in completions2]


def test_multibyte_script():
    """ `jedi.Script` must accept multi-byte string source. """
    try:
        code = u("import datetime; datetime.d")
        comment = u("# multi-byte comment あいうえおä")
        s = (u('%s\n%s') % (code, comment)).encode('utf-8')
    except NameError:
        pass  # python 3 has no unicode method
    else:
        assert len(Script(s, 1, len(code)).completions())


def test_goto_definition_at_zero():
    """At zero usually sometimes raises unicode issues."""
    assert Script("a", 1, 1).goto_definitions() == []
    s = Script("str", 1, 1).goto_definitions()
    assert len(s) == 1
    assert list(s)[0].description == 'class str'
    assert Script("", 1, 0).goto_definitions() == []


def test_complete_at_zero():
    s = Script("str", 1, 3).completions()
    assert len(s) == 1
    assert list(s)[0].name == 'str'

    s = Script("", 1, 0).completions()
    assert len(s) > 0



"""
This is a module that imports the *standard library* unittest,
despite there being a local "unittest" module. It specifies that it
wants the stdlib one with the ``absolute_import`` __future__ import.

The twisted equivalent of this module is ``twisted.trial._synctest``.
"""
from __future__ import absolute_import

import unittest


class Assertions(unittest.TestCase):
    pass



import jedi
from os.path import dirname, join


def test_namespace_package():
    sys_path = [join(dirname(__file__), d)
                for d in ['namespace_package/ns1', 'namespace_package/ns2']]

    def script_with_path(*args, **kwargs):
        return jedi.Script(sys_path=sys_path, *args, **kwargs)

    # goto definition
    assert script_with_path('from pkg import ns1_file').goto_definitions()
    assert script_with_path('from pkg import ns2_file').goto_definitions()
    assert not script_with_path('from pkg import ns3_file').goto_definitions()

    # goto assignment
    tests = {
        'from pkg.ns2_folder.nested import foo': 'nested!',
        'from pkg.ns2_folder import foo': 'ns2_folder!',
        'from pkg.ns2_file import foo': 'ns2_file!',
        'from pkg.ns1_folder import foo': 'ns1_folder!',
        'from pkg.ns1_file import foo': 'ns1_file!',
        'from pkg import foo': 'ns1!',
    }
    for source, solution in tests.items():
        ass = script_with_path(source).goto_assignments()
        assert len(ass) == 1
        assert ass[0].description == "foo = '%s'" % solution

    # completion
    completions = script_with_path('from pkg import ').completions()
    names = [str(c.name) for c in completions]  # str because of unicode
    compare = ['foo', 'ns1_file', 'ns1_folder', 'ns2_folder', 'ns2_file',
               'pkg_resources', 'pkgutil', '__name__', '__path__',
               '__package__', '__file__', '__doc__']
    # must at least contain these items, other items are not important
    assert set(compare) == set(names)

    tests = {
        'from pkg import ns2_folder as x': 'ns2_folder!',
        'from pkg import ns2_file as x': 'ns2_file!',
        'from pkg.ns2_folder import nested as x': 'nested!',
        'from pkg import ns1_folder as x': 'ns1_folder!',
        'from pkg import ns1_file as x': 'ns1_file!',
        'import pkg as x': 'ns1!',
    }
    for source, solution in tests.items():
        for c in script_with_path(source + '; x.').completions():
            if c.name == 'foo':
                completion = c
        solution = "foo = '%s'" % solution
        assert completion.description == solution


def test_nested_namespace_package():
    code = 'from nested_namespaces.namespace.pkg import CONST'

    sys_path = [dirname(__file__)]

    script = jedi.Script(sys_path=sys_path, source=code, line=1, column=45)

    result = script.goto_definitions()

    assert len(result) == 1



import os
from glob import glob
import sys

import pytest

from jedi.evaluate import sys_path
from jedi import Script


def test_paths_from_assignment():
    def paths(src):
        script = Script(src, path='/foo/bar.py')
        expr_stmt = script._get_module_node().children[0]
        return set(sys_path._paths_from_assignment(script._get_module(), expr_stmt))

    assert paths('sys.path[0:0] = ["a"]') == set(['/foo/a'])
    assert paths('sys.path = ["b", 1, x + 3, y, "c"]') == set(['/foo/b', '/foo/c'])
    assert paths('sys.path = a = ["a"]') == set(['/foo/a'])

    # Fail for complicated examples.
    assert paths('sys.path, other = ["a"], 2') == set()


# Currently venv site-packages resolution only seeks pythonX.Y/site-packages
# that belong to the same version as the interpreter to avoid issues with
# cross-version imports.  "venvs/" dir contains "venv27" and "venv34" that
# mimic venvs created for py2.7 and py3.4 respectively.  If test runner is
# invoked with one of those versions, the test below will be run for the
# matching directory.
CUR_DIR = os.path.dirname(__file__)
VENVS = list(glob(
    os.path.join(CUR_DIR, 'sample_venvs/venv%d%d' % sys.version_info[:2])))


@pytest.mark.parametrize('venv', VENVS)
def test_get_venv_path(venv):
    pjoin = os.path.join
    venv_path = sys_path.get_venv_path(venv)

    site_pkgs = (glob(pjoin(venv, 'lib', 'python*', 'site-packages')) +
                 glob(pjoin(venv, 'lib', 'site-packages')))[0]
    ETALON = [
        pjoin('/path', 'from', 'egg-link'),
        pjoin(site_pkgs, '.', 'relative', 'egg-link', 'path'),
        site_pkgs,
        pjoin(site_pkgs, 'dir-from-foo-pth'),
    ]

    # Ensure that pth and egg-link paths were added.
    assert venv_path[:len(ETALON)] == ETALON

    # Ensure that none of venv dirs leaked to the interpreter.
    assert not set(sys.path).intersection(ETALON)

    # Ensure that "import ..." lines were ignored.
    assert pjoin('/path', 'from', 'smth.py') not in venv_path
    assert pjoin('/path', 'from', 'smth.py:extend_path') not in venv_path



if isinstance(i, str):
    #? str()
    i

if isinstance(j, (str, int)):
    #? str() int()
    j

while isinstance(k, (str, int)):
    #? str() int()
    k

if not isinstance(k, (str, int)):
    #? 
    k

while not isinstance(k, (str, int)):
    #? 
    k

assert isinstance(ass, int)
#? int()
ass

assert isinstance(ass, str)
assert not isinstance(ass, int)

if 2:
    #? str()
    ass

# -----------------
# invalid arguments
# -----------------

if isinstance(wrong, str()):
    #?
    wrong

# -----------------
# in functions
# -----------------

import datetime


def fooooo(obj):
    if isinstance(obj, datetime.datetime):
        #? datetime.datetime()
        obj


def fooooo2(obj):
    if isinstance(obj, datetime.date):
        return obj
    else:
        return 1

a
# In earlier versions of Jedi, this returned both datetime and int, but now
# Jedi does flow checks and realizes that the top return isn't executed.
#? int()
fooooo2('')


def isinstance_func(arr):
    for value in arr:
        if isinstance(value, dict):
            # Shouldn't fail, even with the dot.
            #? 17 dict()
            value.
        elif isinstance(value, int):
            x = value
            #? int()
            x

# -----------------
# Names with multiple indices.
# -----------------

class Test():
    def __init__(self, testing):
        if isinstance(testing, str):
            self.testing = testing
        else:
            self.testing = 10

    def boo(self):
        if isinstance(self.testing, str):
            # TODO this is wrong, it should only be str.
            #? str() int()
            self.testing
            #? Test()
            self

# -----------------
# Syntax
# -----------------

#?
isinstance(1, int())
